--[[
	author: Zax
	content:
		globales: SCPLUS_
		functions: ScreenPlus.
		XML: Scplus_
		XML UI: Scplus_ui_
		main UI frame: Scplus_ui_mainFrame
		event frame: Scplus_ui_mainFrame
		XML config: Scplus_cfg_
	todo : 
--]]

---------------------------------------------------------------------------------------
------------------------------------- VARIABLES ---------------------------------------
ScreenPlus = {} -- prefixe fonctions
local addonName = ...
	
local addonVar = { -- 3 niveaux max
	["name"] = addonName, -- nécessaire
	["version"] = GetAddOnMetadata(addonName, "Version"), -- nécessaire
	["chatColor"] = "96A33A", -- nécessaire
	["slashCmd"] = "screenplus", -- nécessaire
	["settingsPanel"] = "ScreenPlusCfgFrame", -- nom du frame de settings : laisser vide si utilisation du frame de WoW
}

local framerateWasVisible = false
local photoProcess = false -- pour n'utiliser les resultats d'events qu'avec cet addon
local cameraOn = false -- flag pour modification/restauration des CVars
local savedUserCVars = {} -- pour sauvegarder l'état des CVars du user avant modification

function ScreenPlus.getLocal(key) -- pour traduction (voir localization.lua) : ne pas deplacer !!!
	if (SCPLUS_LOCAL_TBL[key] == nil) then return key else return SCPLUS_LOCAL_TBL[key] end
end

----------------------------------------- saved et init
SCPLUS_SAVED = {};

function ScreenPlus.initData()
	local tbl = {
		["version"] = addonVar["version"],
		["debug"] = 0,
		["warningTxt"] = 1,
		["warningSound"] = 1,
		["warningChans"] = { -- activation des alertes sur chans
			["guild"] = {["chanName"] = "GUILD", ["abbrev"] = "G", ["on"] = 1},
			["officer"] = {["chanName"] = "OFFICER", ["abbrev"] = "OFF", ["on"] = 1},
			["party"] = {["chanName"] = "PARTY", ["abbrev"] = "P", ["on"] = 1},
			["instance"] = {["chanName"] = "INSTANCE_CHAT", ["abbrev"] = "I", ["on"] = 1},
			["raid"] = {["chanName"] = "RAID", ["abbrev"] = "R", ["on"] = 1},
			["w"] = {["chanName"] = "WHISPER", ["abbrev"] = "W", ["on"] = 1},
			["wbnet"] = {["chanName"] = "BN_WHISPER", ["abbrev"] = "WB", ["on"] = 1},
			["say"] = {["chanName"] = "CHANNEL", ["abbrev"] = "S", ["on"] = 1},
			["channel"] = {["chanName"] = "CHANNEL", ["abbrev"] = "C", ["on"] = 1},
		},
		["display"] = {
			["ui"] = 0,
			["ownName"] = 1,
			["playersName"] = 1,
			["guildsName"] = 0,
			["npcsName"] = 0,
			["petsName"] = 0,
			["selectionCircle"] = 1,
		},
		["captionType"] = 2, -- 0 : aucune, 1 : full et 2 : mini
		["captionFullHeight"] = 100,
		["captionCoords"] = 0,
		["captionHorodate"] = 0,
		["captionColor"] = {
			["r"] = 0.9,
			["g"] = 0.8,
			["b"] = 0.1,
			["a"] = 1,
		},
		["cameraDistance"] = 1.9,
		["screenshotFormat"] = "jpeg",
		["screenshotQuality"] = 3,
	}
	
	tbl["cameraDistance"] = C_CVar.GetCVarDefault("cameraDistanceMaxZoomFactor") -- 1 - 1.9 - 2.6
	tbl["screenshotFormat"] = C_CVar.GetCVarDefault("screenshotFormat") -- "jpeg, tga" ("png" disabled)
	tbl["screenshotQuality"] = C_CVar.GetCVarDefault("screenshotQuality")
	
	return tbl
end

local addonCVars = { -- CVars utilisées et modifiées (regroupées par genre) : valeurs par défaut
	["bubbles"] = {
		["chatBubbles"] = 1,
		["chatBubblesParty"] = 1,
	},
	["selection"] = {
		["ObjectSelectionCircle"] = 1,
	},
	["own"] = {
		["nameplateShowAll"] = 0,
		["nameplateShowSelf"] = 1,
		["UnitNameOwn"] = 0,
	},
	["players"] = {
		["nameplateShowAll"] = 0,
		["UnitNameFriendlyPlayerName"] = 1,
		["UnitNameEnemyPlayerName"] = 1,
		["UnitNamePlayerGuild"] = 1,
		--["UnitNameGuildTitle"] = 1,
		--["UnitNamePlayerPVPTitle"] = 1,
	},
	["npcs"] = {
		["nameplateShowAll"] = 0,
		["UnitNameNPC"] = 0,
		["UnitNameFriendlySpecialNPCName"] = 1,
		["UnitNameHostileNPC"] = 1,
		["UnitNameInteractiveNPC"] = 1,
		["UnitNameNonCombatCreatureName"] = 0,
	},
	["pets"] = {
		["nameplateShowAll"] = 0,
		["UnitNameFriendlyGuardianName"] = 1,
		["UnitNameFriendlyMinionName"] = 1,
		["UnitNameFriendlyPetName"] = 1,
		["UnitNameFriendlyTotemName"] = 1,
		["UnitNameEnemyGuardianName"] = 1,
		["UnitNameEnemyMinionName"] = 1,
		["UnitNameEnemyPetName"] = 1,
		["UnitNameEnemyTotemName"] = 1,
	},
}

function ScreenPlus.getSounds(action)
	local sounds = {
		["guild"] = {["id"] = 39, ["name"] = "Divine Shield"},
		["officer"] = {["id"] = 39, ["name"] = "Divine Shield"},
		
		["party"] = {["id"] = 1204, ["name"] = "PutDownGems"},
		["instance"] = {["id"] = 1204, ["name"] = "PutDownGems"},
		["raid"] = {["id"] = 1204, ["name"] = "PutDownGems"},
		
		["w"] = {["id"] = 5274, ["name"] = "AuctionWindowOpen"},
		["wbnet"] = {["id"] = 5274, ["name"] = "AuctionWindowOpen"},
		
		["channel"] = {["id"] = 6555, ["name"] = "ShaysBell"},
		
		["other"] = {["id"] = 3081, ["name"] = "TellMessage"},
	}
--[[
		["party"] = {["id"] = 1210, ["name"] = "PutDownRing"},
		["raid"] = {["id"] = 2560, ["name"] = "SPELL_Shadow_Cast"},
--]]

	if (sounds[action] == nil) then action = "other" end -- son générique (secu)
	return sounds[action]["id"]
end

---------------------------------------------------------------------------------------
--------------------------------------- EVENTS ----------------------------------------
function ScreenPlus.eventHandler(self, event, ...)
	if (event == "PLAYER_LOGIN") then
		ScreenPlus.chargement();
	elseif (event == "PLAYER_LOGOUT") then
		if (not UIParent:IsVisible() or UIParent:GetAlpha() < 1) then ScreenPlus.cameraModeOff() end -- secu si deco pendant processus capture
	elseif (event == "SCREENSHOT_FAILED") then
		ScreenPlus.photoRate()
	elseif (event == "SCREENSHOT_SUCCEEDED") then
		ScreenPlus.photoPrise()
		
	-- communication : envoyé ou reçu
	-- arg2 = sender, arg4 = channel name, arg8 = channel ID (, arg13 = bnet sender ID)
	elseif (event == "CHAT_MSG_CHANNEL") then
		ScreenPlus.warningMsg("channel", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_SAY" or event == "CHAT_MSG_YELL") then
		ScreenPlus.warningMsg("say", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_GUILD" or event == "CHAT_MSG_GUILD_ACHIEVEMENT") then
		ScreenPlus.warningMsg("guild", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_OFFICER") then
		ScreenPlus.warningMsg("officer", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_PARTY" or event == "CHAT_MSG_PARTY_LEADER") then
		ScreenPlus.warningMsg("party", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_INSTANCE_CHAT" or event == "CHAT_MSG_INSTANCE_CHAT_LEADER") then
		ScreenPlus.warningMsg("instance", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") then
		ScreenPlus.warningMsg("raid", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_WHISPER") then
		ScreenPlus.warningMsg("w", select(2, ...), select(4, ...), select(8, ...))
	elseif (event == "CHAT_MSG_BN_WHISPER") then
		ScreenPlus.warningMsg("wbnet", select(2, ...), select(4, ...), select(8, ...))
	end
end

---------------------------------------------------------------------------------------
--------------------------------- CHARGEMENT & BINDINGS -------------------------------
function ScreenPlus.chargement()
	------------------------------- slash commandes
	SlashCmdList["SCPLUS_CMD"] = ScreenPlus.slashCmd
		SLASH_SCPLUS_CMD1 = "/screenplus"
		SLASH_SCPLUS_CMD2 = "/scplus"
		
	------------------------------- bindings
	BINDING_HEADER_SCPLUS_TITLE = addonName
	BINDING_NAME_SCPLUSPHOTOMODE = ScreenPlus.getLocal("Show / hide main interface")
	BINDING_NAME_SCPLUSSHOOT = ScreenPlus.getLocal("Take a screen snapshot")
	BINDING_NAME_SCPLUSUI = ScreenPlus.getLocal("Display settings")
	
	------------------------------- settings
	if (ScreenPlus.isEmpty(SCPLUS_SAVED)) then
		SCPLUS_SAVED = ScreenPlus.initData()
		ScreenPlus.testPrint("=== no lua file : INIT SETTINGS ===")
	else
		local tbl = ScreenPlus.initData()
		if (ScreenPlus.tableKeyDiff(tbl, SCPLUS_SAVED, "") ~= nil) then
			SCPLUS_SAVED = ScreenPlus.copyTable(tbl, false)
			ScreenPlus.testPrint("=== diff saved data : INIT SETTINGS ===")
		end
	end
	
	SetCVar("cameraDistanceMaxZoomFactor", ScreenPlus.nombreParDefaut(SCPLUS_SAVED["cameraDistance"], 1.9, 1, 2.6))
	SetCVar("screenshotFormat", SCPLUS_SAVED["screenshotFormat"])
	SetCVar("screenshotQuality", SCPLUS_SAVED["screenshotQuality"])
	ScreenPlus.settingsPanelCreate() -- à faire au chargement
	
	------------------------------- suite
	local str = ScreenPlus.getLocal("loaded. Enter /screenplus to display settings.")
	
	if (ScreenPlus.debug()) then
		SetCVar("scriptErrors", 1)
		--ScreenPlus.echoTable(SCPLUS_SAVED, true, "   ")
		str = str..ScreenPlus.getTxtColor(" -- DEBUG ON.", "orange")
	end
	ScreenPlus.echoChatAppl(str, true, nil)
end

---------------------------------------------------------------------------------------
-------------------------------------- SLASH CMD --------------------------------------
function ScreenPlus.slashCmd(arg, execLevel)
	ScreenPlus.slashCmd_Lib(arg, execLevel)
end

function ScreenPlus.slashCmd_getList(arg2) -- liste des commandes-slash (voir .lib)
	-- .slashCmd_showList(dontPrint) -- print la liste en print, ou la renvoit en chaine avec dontPrint à true (pour frame cfg)
	local tbl = {
		["user"] = {
			{
				["cmd"] = {"ui", "cfg", "config"},
				["func"] = function() ScreenPlus.showCfgUI() end,
				["label"] = ScreenPlus.getLocal("Show / hide main interface"),
			},
			{
				["cmd"] = {"hide"},
				["func"] = function() ScreenPlus.cameraModeOn() end,
				["label"] = ScreenPlus.getLocal("enter camera mode"),
			},
			{
				["cmd"] = {"show"},
				["func"] = function() ScreenPlus.cameraModeOff() end,
				["label"] = ScreenPlus.getLocal("exit camera mode"),
			},
			{
				["cmd"] = {"shoot"},
				["func"] = function() ScreenPlus.captureEcran() end,
				["label"] = ScreenPlus.getLocal("take a screen capture"),
			},
			{
				["cmd"] = {"warning"},
				["type"] = "guild | officer | party | instance | raid | w | wbnet",
				["func"] = function() ScreenPlus.warningSwitch(arg2) end,
				["label"] = ScreenPlus.getLocal("turn on/off alerts for defined channel"),
			},
			{
				["cmd"] = {"reset"},
				["func"] = function() ScreenPlus.reset() end,
				["label"] = ScreenPlus.getLocal("reset all settings and CVars to default"),
			},
		},
		["dev"] = {
			{
				["cmd"] = {"debug"},
				["func"] = function() ScreenPlus.toggleDebug() end,
				["label"] = "toggle debug",
			},
			{
				["cmd"] = {"settings"},
				["func"] = function() ScreenPlus.printAddonSettings() end,
				["label"] = ScreenPlus.getLocal("print addon settings"),
			},
			{
				["cmd"] = {"test"},
				["func"] = function() ScreenPlus.fcnTest(arg2) end,
				["label"] = "DEV test with 1 argument",
			},
			{
				["cmd"] = {"cvars"},
				["func"] = function() ScreenPlus.printCVars() end,
				["label"] = "print CVars",
			},
			{
				["cmd"] = {"snd", "play"},
				["func"] = function() ScreenPlus.testSound(arg2) end,
				["label"] = "play passed sound as argument",
			},
			{
				["cmd"] = {"help"},
				["func"] = function() ScreenPlus.help() end,
				["label"] = "compatibility command",
			},
		},
		["DEFAULTCMD"] = "cfg", -- commande par défaut
		["DEFAULTARG"] = "help", -- argument par défaut d'une commande
	}
	return tbl
end

function ScreenPlus.reset()
	local cfgUIvisible = Scplus_ui_settings:IsVisible();
	ScreenPlus.closeCfgUI()
	UIParent:SetAlpha(1) -- secu dev
	--------------- UI settings
	wipe(SCPLUS_SAVED)
	SCPLUS_SAVED = ScreenPlus.initData()
	--------------- CVars
	ScreenPlus.cvarsReset()
	---------------
	if (cfgUIvisible) then
		ScreenPlus.showCfgUI() -- re-affichage des options si necessaire
	else
		ScreenPlus.echoChatAppl(ScreenPlus.getLocal("all settings and CVars reset to defaults."), false, "yellow")
	end;
end

function ScreenPlus.help()
	if (Settings ~= nil) then -- meilleur test que .isRetail()
		Settings.OpenToCategory(addonName) -- wow10
	else
		InterfaceAddOnsList_Update()
		InterfaceOptionsFrame_OpenToCategory(addonName) -- wow9
	end
end

function ScreenPlus.fcnTest(arg) --$$$
	print("------------------------------------------")
	print(date("%Y.%m.%d %H:%M:%S", GetServerTime()))
	local version, build, datevers, tocversion = GetBuildInfo()
	print(version, build, datevers, "toc = "..tocversion)
	print("------------------------------------------")
	--print("getCoords()", ScreenPlus.getCoords())
end

function ScreenPlus.testSound(idSound) -- pour dev
	ScreenPlus.soundTest(idSound)
end

function ScreenPlus.printAddonSettings()
	ScreenPlus.echoChatAppl("settings", true, "grey")
	
	for key, value in pairs(SCPLUS_SAVED) do
		if (type(value) == "table") then
			ScreenPlus.echoChat("    "..key..":", "SYSTEM", true)
			if (key == "warningChans") then
				for subkey, subvalue in pairs(value) do
					ScreenPlus.echoChat("        "..subkey..": "..subvalue["on"], "SYSTEM", true)
				end
			else
				for subkey, subvalue in pairs(value) do
					if (type(subvalue) == "table") then subvalue = "TABLE" end
					ScreenPlus.echoChat("        "..subkey..": "..subvalue, "SYSTEM", true);
				end
			end
		else
			ScreenPlus.echoChat("    "..tostring(key)..": "..tostring(value), "SYSTEM", true)
		end
	end
end

function ScreenPlus.toggleDebug() -- pour dev
	if (ScreenPlus.isTrue(SCPLUS_SAVED["debug"])) then
		SCPLUS_SAVED["debug"] = 0
	else
		SCPLUS_SAVED["debug"] = 1
	end
	ScreenPlus.echoChatAppl("debug "..ScreenPlus.booleanToON(ScreenPlus.debug())..".", true, "orange")
end

function ScreenPlus.printCVars()
	ScreenPlus.echoChatAppl("CVars", true, "grey")
	ScreenPlus.echoChat("        cameraDistance: "..GetCVar("cameraDistanceMaxZoomFactor"), "SYSTEM", true)
	ScreenPlus.echoChat("        screenshotFormat: "..GetCVar("screenshotFormat"), "SYSTEM", true)
	ScreenPlus.echoChat("        screenshotQuality: "..GetCVar("screenshotQuality"), "SYSTEM", true)
	
	ScreenPlus.echoChatAppl("CVars during screen mode", true, "grey")
	for key, value in pairs(addonCVars) do
		if type(value) == "table" then
			for cvarName, cvarValue in pairs(value) do
				ScreenPlus.echoChat("        "..cvarName..": "..cvarValue, "SYSTEM", true)
			end
		else
			ScreenPlus.echoChat("    "..tostring(key)..": "..tostring(value), "SYSTEM", true)
		end
	end
end

function ScreenPlus.warningSwitch(channel) -- activation ou désactivation des alertes pour un type de message reçu
	if (type(SCPLUS_SAVED["warningChans"][channel]) == "table") then
		local currentState = SCPLUS_SAVED["warningChans"][channel]["on"]
		local newState = ScreenPlus.inversionValeur(currentState)
		SCPLUS_SAVED["warningChans"][channel]["on"] = newState
		local str = ScreenPlus.getLocal("message deactivated.")
		if (ScreenPlus.isTrue(newState)) then str = ScreenPlus.getLocal("message activated.") end
		ScreenPlus.echoChatAppl(ScreenPlus.getLocal("alert for").." '"..channel.."' "..str, "false", "")
	end
end

---------------------------------------------------------------------------------------
--------------------------------- MODE CAMERA ON/OFF ---------------------------------------------------------------------------------------
function ScreenPlus.showHideUI() -- switch (bind)
	if (UIParent:IsVisible() and UIParent:GetAlpha() == 1) then ScreenPlus.cameraModeOn() else ScreenPlus.cameraModeOff() end
end

function ScreenPlus.cameraModeOn()
	cameraOn = true
	ScreenPlus.mainUIaffecte(true) -- sauvegarde des CVars et modif UI
end

function ScreenPlus.cameraModeOff()
	cameraOn = false
	ScreenPlus.mainUIrestaure()
end

---------------------------------------------------------------------------------------
------------------------- PLAYAGE DE SONS (quand UI masquée) --------------------------
function ScreenPlus.warningMsg(msgKind, sender, chanName, chanIndex) -- message reçu ou envoyé (event)
	if (ScreenPlus.isTrue(SCPLUS_SAVED["warningSound"]) or ScreenPlus.isTrue(SCPLUS_SAVED["warningTxt"])) then -- prefs
		local playerName = ScreenPlus.getPlayerFullName()
		local senderName = ScreenPlus.getSenderFullName(sender)
		if ((senderName ~= playerName) or ScreenPlus.debug()) then
			if (ScreenPlus.debug()) then
				ScreenPlus.echoChatAppl(date("%Hh %M' %S''", GetServerTime())..": msgKind="..ScreenPlus.nilSiNul(msgKind).." état="..SCPLUS_SAVED["warningChans"][msgKind]["on"].." --- sender="..ScreenPlus.nilSiNul(sender).." --- chanName="..ScreenPlus.nilSiNul(chanName).." --- chanIndex="..ScreenPlus.nilSiNul(chanIndex), true, "grey")
			end
			if (cameraOn and ScreenPlus.isTrue(SCPLUS_SAVED["warningChans"][msgKind]["on"])) then
				--ScreenPlus.testPrint("snd="..SCPLUS_SAVED["warningSound"].." txt="..SCPLUS_SAVED["warningTxt"])
				if (ScreenPlus.isTrue(SCPLUS_SAVED["warningSound"])) then ScreenPlus.warningPlaySound(msgKind) end
				if (ScreenPlus.isTrue(SCPLUS_SAVED["warningTxt"])) then ScreenPlus.warningShowTxt(msgKind) end
			end
		end
	end
end

function ScreenPlus.warningPlaySound(actionName)
	--ScreenPlus.testPrint("----> playage son action="..ScreenPlus.nilSiNul(actionName))
	PlaySound(ScreenPlus.getSounds(actionName))
end

function ScreenPlus.warningShowTxt(msgKind)
	------------------------------- peuplement frame
	local chanName
	local r, g, b = 0.1, 0.1, 0.7
	if (type(SCPLUS_SAVED["warningChans"][msgKind]) == "table") then
		Scplus_ui_warningFrame_Text:SetText(SCPLUS_SAVED["warningChans"][msgKind]["abbrev"])
		chanName = SCPLUS_SAVED["warningChans"][msgKind]["chanName"]
		if (ChatTypeInfo[chanName] ~= nil) then
			r, g, b = Chat_GetChannelColor(ChatTypeInfo[chanName])
		end
	else
		Scplus_ui_warningFrame_Text:SetText("")
	end
	Scplus_ui_warningFrame_Bk:SetColorTexture(r, g, b)
	------------------------------- affichage frame
	Scplus_ui_warningFrame:Show()
	C_Timer.After(2, function() -- secondes
		Scplus_ui_warningFrame:Hide()
	end)
end

function ScreenPlus.getPlayerFullName()
	local playerName, realm = UnitFullName("player") -- nom condensé du server (sans espace)
	return (playerName.."-"..realm)
end

function ScreenPlus.getSenderFullName(sender) -- renvoit le nom du sender avec le serveur (si absent)
	if (sender ~= nil) then
		local senderName, realm = strsplit("-", sender, 2)
		if (senderName == sender) then
			return senderName.."-"..GetRealmName() -- ajout du serveur si absent
		else
			return sender
		end
	else
		return ""
	end
end

---------------------------------------------------------------------------------------
--------------------------------------- CAPTURE ---------------------------------------
function ScreenPlus.captureEcran()
	photoProcess = true
	if (not cameraOn) then ScreenPlus.mainUIaffecte(not Scplus_ui_settings:IsVisible()) end -- sauvegarde des CVars et modif UI si l'UI de config n'est pas visible
	ScreenPlus.showCaption()
	Screenshot() -- capture : termine le bloc : la suite se fait par capture d'evenements
end


function ScreenPlus.captionsHide() -- masque tous les frames de légende
	Scplus_ui_mainFrame:Hide()
	Scplus_ui_miniCaptionFrame:Hide()
end

function ScreenPlus.showCaption() -- affichage légende full ou mini ou aucune
	ScreenPlus.captionsHide() -- ménage
	if (ScreenPlus.strToNumber(SCPLUS_SAVED["captionType"]) == 1) then
		-------------- légende complete
		Scplus_ui_caption:SetTextColor(SCPLUS_SAVED["captionColor"]["r"],SCPLUS_SAVED["captionColor"]["g"],SCPLUS_SAVED["captionColor"]["b"],SCPLUS_SAVED["captionColor"]["a"])
		Scplus_ui_caption:SetText(ScreenPlus.creationCaption())
		Scplus_ui_mainFrame:Show()
	elseif (ScreenPlus.strToNumber(SCPLUS_SAVED["captionType"]) == 2) then
		-------------- mini légende
		Scplus_ui_miniCaption:SetText(ScreenPlus.creationCaption("mini"))
		Scplus_ui_miniCaptionFrame:Show()
	else
		-------------- pas de légende
	end
end

function ScreenPlus.creationCaption(genre) -- génération du texte de la légende complète
	local sep = "\n"
	if (genre == "mini") then sep = " - " end
	local caption = ScreenPlus.getZoneStr()
	if (ScreenPlus.isTrue(SCPLUS_SAVED["captionCoords"])) then caption = caption.." ("..ScreenPlus.getCoords()..")" end
	if (ScreenPlus.isTrue(SCPLUS_SAVED["captionHorodate"])) then caption = caption..sep..date("%Y.%m.%d %H:%M", GetServerTime()) end
	return caption
end

function ScreenPlus.getZoneStr() -- renvoit la zone et la sub-zone du joueur en chaine
	local subZone = GetSubZoneText()
	if (subZone ~= nil and subZone ~= "" and subZone ~= GetZoneText()) then
		return GetZoneText().." - "..subZone
	else
		return GetZoneText()
	end
end

function ScreenPlus.getCoords() -- renvoit la position du joueur en chaine
	local positionStr = "unknown"
	local h,v = ScreenPlus.getMapCoords()
	if (h ~= "") then
		h = ScreenPlus.round(h * 100)
		v = ScreenPlus.round(v * 100)
		if (h > 0 and v > 0) then positionStr = "h"..h..", v"..v end
	end
	return positionStr
end

function ScreenPlus.getMapCoords() -- renvoit la position du joueur, ou ""
	local position
	local map = C_Map.GetBestMapForUnit("player")
	if (map ~= nil) then
		position = C_Map.GetPlayerMapPosition(map, "player")
		return position:GetXY()
	else
		return ""
	end
end

---------------------------------------------------------------------------------------
------------------------------------ SUITE CAPTURE ------------------------------------
function ScreenPlus.photoPrise()
	if (photoProcess) then
		PlaySoundFile("Interface\\AddOns\\ScreenPlus\\stuff\\camera.mp3")
		ScreenPlus.echoChatAppl(ScreenPlus.getLocal("screen capture saved in folder").." '"..ScreenPlus.getCheminDossScreen().."'.", false, "gray")
		ScreenPlus.suiteCapture()
	end;
end

function ScreenPlus.photoRate()
	if (photoProcess) then
		PlaySound(IG_QUEST_CANCEL) -- "igQuestFailed"
		ScreenPlus.echoChatAppl(ScreenPlus.getLocal("screenshot failed!"), false, "red")
		ScreenPlus.suiteCapture()
	end;
end

function ScreenPlus.suiteCapture() -- re-affichage
	photoProcess = false
	if (not Scplus_ui_settings:IsVisible()) then
		ScreenPlus.captionsHide()
		if (not cameraOn) then ScreenPlus.mainUIrestaure() end
	end
end

function ScreenPlus.getCheminDossScreen()
	if (ScreenPlus.isRetail()) then
		return "World of Warcraft\\_retail_\\Screenshots\\"
	else
		return "World of Warcraft\\_classic_\\Screenshots\\"
	end
end

---------------------------------------------------------------------------------------
------------------------------------ FCNS INTERNES ------------------------------------
function ScreenPlus.mainUIaffecte(sauvegardeBooleen) -- modification des CVars suivant prefs
	------------------------------------ main UI
	if (ScreenPlus.debug()) then UIParent:SetAlpha(0.5) else UIParent:Hide() end
	framerateWasVisible = FramerateText:IsVisible()
	if (framerateWasVisible) then ToggleFramerate() end
	------------------------------------ CVars
	if (not ScreenPlus.isTrue(sauvegardeBooleen)) then sauvegardeBooleen = false end
	if (sauvegardeBooleen) then ScreenPlus.cvarsSauve() end
	-------------- bubbles
	for cvarName, cvarValue in pairs(addonCVars["bubbles"]) do
		if (GetCVar(cvarName) ~= nil) then SetCVar(cvarName, 0) end
	end
	-------------- suivant prefs (SCPLUS_SAVED)
	local prefVal = ScreenPlus.strToNumber(SCPLUS_SAVED["display"]["ownName"]) -- nom checkbox UI
	for cvarName, cvarValue in pairs(addonCVars["own"]) do
		if (GetCVar(cvarName) ~= nil) then SetCVar(cvarName, prefVal) end
	end
	prefVal = ScreenPlus.strToNumber(SCPLUS_SAVED["display"]["playersName"]) -- nom checkbox UI
	for cvarName, cvarValue in pairs(addonCVars["players"]) do
		if (GetCVar(cvarName) ~= nil) then SetCVar(cvarName, prefVal) end
	end
	prefVal = ScreenPlus.strToNumber(SCPLUS_SAVED["display"]["npcsName"]) -- nom checkbox UI
	for cvarName, cvarValue in pairs(addonCVars["npcs"]) do
		if (GetCVar(cvarName) ~= nil) then SetCVar(cvarName, prefVal) end
	end
--[[
	prefVal = ScreenPlus.strToNumber(SCPLUS_SAVED["display"]["guildsName"])
	for cvarName, cvarValue in pairs(addonCVars["guild"]) do
		SetCVar(cvarName, prefVal)
	end
--]]
	prefVal = ScreenPlus.strToNumber(SCPLUS_SAVED["display"]["petsName"]) -- nom checkbox UI
	for cvarName, cvarValue in pairs(addonCVars["pets"]) do
		if (GetCVar(cvarName) ~= nil) then SetCVar(cvarName, prefVal) end
	end
	
	prefVal = ScreenPlus.strToNumber(SCPLUS_SAVED["display"]["selectionCircle"])
	for cvarName, cvarValue in pairs(addonCVars["selection"]) do
		if (GetCVar(cvarName) ~= nil) then SetCVar(cvarName, prefVal) end
	end
--[[
	["pvp"] = {
		["UnitNamePlayerPVPTitle"] = 1,
	},
--]]
end

function ScreenPlus.mainUIrestaure()
	if (ScreenPlus.debug()) then UIParent:SetAlpha(1) else UIParent:Show() end
	if (framerateWasVisible) then ToggleFramerate() end
	ScreenPlus.cvarsRestaure()
end
	
function ScreenPlus.cvarsSauve() -- sauvegarde de l'état des CVars du user : savedUserCVars
	ScreenPlus.testPrint("-------------->>> SAUVE")
	wipe(savedUserCVars)
	
	for key, value in pairs(addonCVars) do
		if type(value) == "table" then
			for cvarName, cvarValue in pairs(value) do
				savedUserCVars[cvarName] = GetCVar(cvarName)
			end
		else
			savedUserCVars[key] = GetCVar(key)
		end
	end
end

function ScreenPlus.cvarsRestaure() -- restauration de l'état des CVars du user : savedUserCVars
	ScreenPlus.testPrint("<<<-------------- RESTAURE")
	
	for key, value in pairs(savedUserCVars) do
		SetCVar(key, value)
	end
end

function ScreenPlus.cvarsReset() -- reset des CVars utilisées à leurs valeurs par défaut
	ScreenPlus.testPrint("<<<-------------- CVARS RESET")
	
	wipe(savedUserCVars)
	for key, value in pairs(addonCVars) do
		if type(value) == "table" then
			for cvarName, cvarValue in pairs(value) do
				SetCVar(cvarName, C_CVar.GetCVarDefault(cvarName))
			end
		else
			SetCVar(key, C_CVar.GetCVarDefault(key))
		end
	end
	SetCVar("cameraDistanceMaxZoomFactor", C_CVar.GetCVarDefault("cameraDistanceMaxZoomFactor"))
	SetCVar("screenshotFormat", C_CVar.GetCVarDefault("screenshotFormat"))
	SetCVar("screenshotQuality", C_CVar.GetCVarDefault("screenshotQuality"))
end

function ScreenPlus.cvarsSet(cvarCat, newValeur) -- attribue une valeur (int) à une 'catégorie' de CVars
	local affecteOK = false
	local intVal = ScreenPlus.strToNumber(newValeur)
	if (addonCVars[cvarCat] ~= nil) then
		if type(addonCVars[cvarCat]) == "table" then
			ScreenPlus.testPrint("cvarsSet(): addonCVars["..ScreenPlus.nilSiNul(cvarCat).."][TABLE] --> "..intVal)
			for cvarName, cvarValue in pairs(addonCVars[cvarCat]) do
				addonCVars[cvarCat][cvarName] = intVal
			end
			affecteOK = true
		else
			ScreenPlus.testPrint("cvarsSet(): addonCVars["..ScreenPlus.nilSiNul(cvarCat).."] --> "..intVal)
			addonCVars[cvarCat] = intVal
			affecteOK = true
		end
	end
	
	if (affecteOK) then ScreenPlus.mainUIaffecte(false) end -- effet immédiat SANS sauvegarde
end

---------------------------------------------------------------------------------------
------------------------------------- LIB DEDIEE --------------------------------------
function ScreenPlus.debug() -- raccourci : renvoit true si debug ON
	return ScreenPlus.isTrue(SCPLUS_SAVED["debug"])
end

function ScreenPlus.getVar(nomVar, subLvl1, subLvl2) -- nécessite addonVar
	-- get valeurs de la table locale addonVar
	-- si aucun argument n'est passé, renvoit la table addonVar
	if (addonVar == nil) then return nil end
	
	if (subLvl2 ~= nil) then
		if (addonVar[nomVar][subLvl1][subLvl2] ~= nil) then return addonVar[nomVar][subLvl1][subLvl2] end
	elseif (subLvl1 ~= nil) then
		if (addonVar[nomVar][subLvl1] ~= nil) then return addonVar[nomVar][subLvl1] end
	elseif (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then return addonVar[nomVar] end
	else
		return addonVar
	end
end

function ScreenPlus.setVar(nomVar, valeur) -- nécessite addonVar
	if (addonVar == nil) then return nil end
	if (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then addonVar[nomVar] = valeur end
	end
end

function ScreenPlus.getSavedSettings(perCharacterBoolean) -- pour appel depuis autres fichiers
	if (perCharacterBoolean) then return SCPLUS_SAVED else return SCPLUS_SAVED end
end

---------------------------------------------------------------------------------------
-------------------------------- SETTINGS UI (DEDIE) ----------------------------------
function ScreenPlus.showCfgUI() -- hide/show config UI frame
	local shown = false
	if (Scplus_ui_settings ~= nil) then
		shown = Scplus_ui_settings:IsVisible()
	end
	if (shown) then
		ScreenPlus.closeCfgUI()
	else
		ScreenPlus.settingsPanelShow() -- ScreenPlus_settings.lua (init et/ou update)
		ScreenPlus.cfgUIShown()-- masquage UI
	end
end

function ScreenPlus.closeCfgUI()
	Scplus_ui_mainFrame:Hide()
	Scplus_ui_settings:Hide()
	-- les modifs des prefs sont faites en temps réel depuis l'UI de config
	ScreenPlus.captionsHide()
	if (not cameraOn) then ScreenPlus.mainUIrestaure() end
end

function ScreenPlus.cfgUIShown()
	ScreenPlus.mainUIaffecte(not cameraOn)
end